/***********************************************************
Header file for optical mouse sensor capabilities

William Westrick
Nicole Rodia
************************************************************/

#define a 2.947 //from cad and sensor datasheet
#define b 2.953 //from cad and sensor datasheet
#define c 2.773 //from cad and sensor datasheet
#define d c //same as c

//coefficients for position decoding from sensors
#define a_plus_b ((a)+(b))
#define c_minus_d ((c)-(d))
#define c_plus_d ((c)+(d))
#define a_minus_b ((a)-(b))
#define d_minus_c ((d)-(c))

#define a_plus_b_squared ((a_plus_b)*(a_plus_b))
#define c_minus_d_squared ((c_minus_d)*(c_minus_d))
#define a_plus_b_squared_plus_c_minus_d_squared ((a_plus_b_squared)+(c_minus_d_squared))

#define deltaX_Y_denominator ((float)(2*(a_plus_b_squared_plus_c_minus_d_squared)))
#define omega_denominator ((float)(a_plus_b_squared_plus_c_minus_d_squared))

#define deltaX_X1_coef ((float)(((a_plus_b_squared)+(2*(c)*(c_minus_d)))/(deltaX_Y_denominator)))
#define deltaX_X2_coef ((float)(((a_plus_b_squared)+(2*(d)*(d_minus_c)))/(deltaX_Y_denominator)))
#define deltaX_Y1_minus_Y2_coef ((float)(((a_plus_b)*(c_plus_d))/(deltaX_Y_denominator)))

#define deltaY_X1_coef ((float)(((a_minus_b)*(c_minus_d))/(deltaX_Y_denominator)))
#define deltaY_X2_coef ((float)((-(deltaY_X1_coef))/(deltaX_Y_denominator)))
#define deltaY_Y1_coef ((float)(((2*(a)*(a_plus_b))+(c_minus_d_squared))/(deltaX_Y_denominator)))
#define deltaY_Y2_coef ((float)(((2*(b)*(a_plus_b))+(c_minus_d_squared))/(deltaX_Y_denominator)))

#define omega_X1_minus_X2_coef ((float)((c_minus_d)/(omega_denominator)))
#define omega_Y1_minus_Y2_coef ((float)((a_plus_b)/(omega_denominator)))

/***********************************************************
* PIN NAME MACROS
************************************************************/
#define DD_MOSI PB5
#define DD_MISO PB6
#define DD_SCK  PB7
#define CS1		PB0
#define CS2		PB1
#define	RESET1	PB2
#define	RESET2	PB3

/***********************************************************
* VARIABLES
************************************************************/
signed char x[2], y[2];
unsigned char data, srom_result;
signed long sum_x[2], sum_y[2];
float acc_x[2], acc_y[2];
float xval, xval2, yval, omega;
float absx, absy, absrot;  //absolute x,y,rot since the sensors began reading motion
unsigned char data_ready;
unsigned char squal[2];

char temp_str[50]; //temporary string for serial communications

/***********************************************************
* FUNCTION PROTOTYPES
************************************************************/
//Init the SPI
void SPI_MasterInit(void);
//Transmit two bytes to the specified sensor
char SPI_MasterTransmit(unsigned char, unsigned char, unsigned char);
//Read the motion registers from the specified sensor			
char Read_Data(unsigned char);
//Send the motion data over the uart
void Send_Data(void);
//Write to the SROM for the specified sensor
char SROM(unsigned char);
//Burst mode for the specified sensor
void Burst_Mode(unsigned char, char, char *);
//Setup the optical sensors
void Init_Opt(void);
//calculate the position based off optical sensor readings
void Calc_Pos(void);
//initialize everything related to mouse sensors
void initializeMouseSensors(void);
//read the surface quality from each sensor
void ReadSurfaceQuality(void);
